#include <windows.h>
#include <stdio.h>

int main(int ac, char **av)
{
    HDC dc;
    HFONT font, ofont;
    GLYPHSET *gs;

    dc = CreateCompatibleDC(NULL);
    if(dc == NULL) {
        printf("CreateCompatibleDC failed: %lu\n", GetLastError());
        return 1;
    }

    font = CreateFontW(-16, 0, 0, 0,
                       FW_NORMAL, FALSE, FALSE, FALSE,
                       SHIFTJIS_CHARSET,
                       OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS,
                       DEFAULT_QUALITY,
                       DEFAULT_PITCH,
                       L"\xff2d\xff33 \xff30\x30b4\x30b7\x30c3\x30af");
    if(font == NULL) {
        printf("CreateFontW failed: %lu\n", GetLastError());
        return 1;
    }

    ofont = SelectObject(dc, font);
    if(ofont == NULL) {
        printf("SelectObject failed: %lu\n", GetLastError());
        return 1;
    }

    {
        DWORD gssize = GetFontUnicodeRanges(dc, NULL);
        gs = (GLYPHSET *)malloc(gssize);
        memset(gs, 0, gssize);
        gs->cbThis = gssize;
        GetFontUnicodeRanges(dc, gs);
    }

    SetTextAlign(dc, TA_TOP | TA_LEFT);
    SetTextColor(dc, 0xffffff);
    SetBkColor(dc, 0x000000);

    {
        int i, j, k;
        wchar_t wc;
        char mbs[2];
        BOOL isdef;
        ABC abc;
        int width;
        HBITMAP bmp, obmp;
        BITMAPINFO bmi;
        DWORD buf[16];

        for(i = 0; i < gs->cRanges; i++) {
            for(j = 0; j < gs->ranges[i].cGlyphs; j++) {
                wc = gs->ranges[i].wcLow + j;
                if(WideCharToMultiByte(932, WC_NO_BEST_FIT_CHARS,
                                       &wc, 1,
                                       mbs, 2,
                                       NULL, &isdef) == 0) {
                    printf("WideCharToMultiByte failed: %lu\n", GetLastError());
                    return 1;
                }

                if(! GetCharABCWidthsW(dc, wc, wc, &abc)) {
                    printf("GetCharABCWidths failed: %lu\n", GetLastError());
                    return 1;
                }
                width = abc.abcA + abc.abcB + abc.abcC;

                if(width == 0) {
                    continue;
                }

                bmp = CreateBitmap(width, 16, 1, 1, NULL);
                if(bmp == NULL) {
                    printf("CreateBitmap failed: %lu\n", GetLastError());
                    return 1;
                }

                obmp = SelectObject(dc, bmp);
                TextOutW(dc, 0, 0, &wc, 1);
                SelectObject(dc, obmp);

                memset(&bmi, 0, sizeof(bmi));
                bmi.bmiHeader.biSize = sizeof(bmi.bmiHeader);
                bmi.bmiHeader.biWidth = width;
                bmi.bmiHeader.biHeight = 16;
                bmi.bmiHeader.biPlanes = 1;
                bmi.bmiHeader.biBitCount = 1;
                bmi.bmiHeader.biCompression = BI_RGB;

                if(GetDIBits(dc, bmp, 0, 16, buf, &bmi, DIB_PAL_COLORS) == 0) {
                    printf("GetDIBits failed: %lu\n", GetLastError());
                    return 1;
                }

                DeleteObject(bmp);

                printf("%c %04x ", (isdef ? ' ' : '*'), wc);
                for(k = 15; k >= 0; k--) {
                    printf("%lx ", buf[k]);
                }
                printf("\n");
            }
        }
    }

    SelectObject(dc, ofont);
    DeleteObject(font);
    DeleteDC(dc);

    return 0;
}
