#include <windows.h>
#include <stdio.h>

int main(int ac, char **av)
{
    HDC dc;
    HFONT font, ofont;
    GLYPHSET *gs;

    dc = CreateCompatibleDC(NULL);
    if(dc == NULL) {
        printf("CreateCompatibleDC failed: %lu\n", GetLastError());
        return 1;
    }

    font = CreateFontW(-16, 0, 0, 0,
                       FW_NORMAL, FALSE, FALSE, FALSE,
                       SHIFTJIS_CHARSET,
                       OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS,
                       DEFAULT_QUALITY,
                       DEFAULT_PITCH,
                       L"\xff2d\xff33 \xff30\x30b4\x30b7\x30c3\x30af");
    if(font == NULL) {
        printf("CreateFontW failed: %lu\n", GetLastError());
        return 1;
    }

    ofont = SelectObject(dc, font);
    if(ofont == NULL) {
        printf("SelectObject failed: %lu\n", GetLastError());
        return 1;
    }

    {
        DWORD gssize = GetFontUnicodeRanges(dc, NULL);
        gs = (GLYPHSET *)malloc(gssize);
        memset(gs, 0, gssize);
        gs->cbThis = gssize;
        GetFontUnicodeRanges(dc, gs);
    }

    {
        int i, j;
        wchar_t wc;
        char mbs[2];
        BOOL isdef;
        ABC abc;

        for(i = 0; i < gs->cRanges; i++) {
            for(j = 0; j < gs->ranges[i].cGlyphs; j++) {
                wc = gs->ranges[i].wcLow + j;
                if(WideCharToMultiByte(932, WC_NO_BEST_FIT_CHARS,
                                       &wc, 1,
                                       mbs, 2,
                                       NULL, &isdef) == 0) {
                    printf("WideCharToMultiByte failed: %lu\n", GetLastError());
                    return 1;
                }

                if(! GetCharABCWidthsW(dc, wc, wc, &abc)) {
                    printf("GetCharABCWidths failed: %lu\n", GetLastError());
                    return 1;
                }

                printf("%c %04x %d\n",
                       (isdef ? ' ' : '*'),
                       wc, abc.abcA + abc.abcB + abc.abcC);
            }
        }
    }

    SelectObject(dc, ofont);
    DeleteObject(font);
    DeleteDC(dc);

    return 0;
}
